/*
 *  Copyright 2017 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.plugins.userdirectory.clientsideelement;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Value;

import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.jackrabbit.value.StringValue;

import org.ametys.core.observation.Event;
import org.ametys.core.observation.ObservationManager;
import org.ametys.core.ui.Callable;
import org.ametys.core.util.LambdaUtils;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.jcr.JCRAmetysObject;
import org.ametys.plugins.userdirectory.OrganisationChartPageHandler;
import org.ametys.plugins.userdirectory.observation.ObservationConstants;
import org.ametys.plugins.userdirectory.page.VirtualOrganisationChartPageFactory;
import org.ametys.runtime.i18n.I18nizableText;
import org.ametys.web.clientsideelement.AbstractPageClientSideElement;
import org.ametys.web.repository.page.Page;

/**
 * Client side element for a controller wich set/remove the organisation chart root page
 */
public class SetOrganisationChartRootClientSideElement extends AbstractPageClientSideElement
{
    /** Observer manager. */
    protected ObservationManager _observationManager;

    /** The organisation chart page handler */
    protected OrganisationChartPageHandler _pageHandler;
    
    @Override
    public void service(ServiceManager smanager) throws ServiceException
    {
        super.service(smanager);
        _observationManager = (ObservationManager) smanager.lookup(ObservationManager.ROLE);
        _pageHandler = (OrganisationChartPageHandler) smanager.lookup(OrganisationChartPageHandler.ROLE);
    }
    
    /**
     * Gets the status of the given page
     * @param pageId The page id
     * @return the status of the given page
     */
    @Callable
    public Map<String, Object> getStatus(String pageId)
    {
        Map<String, Object> result = new HashMap<>();
        
        Map<String, Object> parameters = this._script.getParameters();
        
        Page page = _resolver.resolveById(pageId);

        if (page instanceof JCRAmetysObject)
        {
            if (_pageHandler.isOrganisationChartRootPage((JCRAmetysObject) page))
            {
                List<String> i18nParameters = new ArrayList<>();
                i18nParameters.add(page.getTitle());
    
                I18nizableText ed = (I18nizableText) parameters.get("organisation-chart-page-description");
                I18nizableText msg = new I18nizableText(ed.getCatalogue(), ed.getKey(), i18nParameters);
                result.put("organisation-chart-page-title", msg);
                
                ed = (I18nizableText) parameters.get("remove-organisation-chart-page-description");
                msg = new I18nizableText(ed.getCatalogue(), ed.getKey(), i18nParameters);
                result.put("remove-organisation-chart-page-title", msg);
                
                result.put("organisation-chart-page-id", new I18nizableText(page.getId()));
            }
            else if (_pageHandler.getOrganisationChartRootPages(page.getSiteName(), page.getSitemapName()) != null)
            {
                I18nizableText ed = (I18nizableText) parameters.get("organisation-chart-root-already-exist");
                result.put("organisation-chart-root-already-exist", ed);
            }
            else
            {
                List<String> i18nParameters = new ArrayList<>();
                i18nParameters.add(page.getTitle());
    
                I18nizableText ed = (I18nizableText) parameters.get("add-organisation-chart-page-description");
                I18nizableText msg = new I18nizableText(ed.getCatalogue(), ed.getKey(), i18nParameters);
                
                result.put("add-organisation-chart-page-id", new I18nizableText(page.getId()));
                result.put("add-organisation-chart-page-title", msg);
            }
        }
        else
        {
            List<String> noJcrI18nParameters = new ArrayList<>();
            noJcrI18nParameters.add(page.getTitle());

            I18nizableText ed = (I18nizableText) parameters.get("no-jcr-page-description");
            I18nizableText msg = new I18nizableText(ed.getCatalogue(), ed.getKey(), noJcrI18nParameters);
            
            result.put("no-jcr-page-id", new I18nizableText(page.getId()));
            result.put("no-jcr-page-title", msg);
        }
        
        return result;
    }
    
    /**
     * Sets the given page as the root of a organization chart
     * @param pageId The id of the page
     * @return A result map
     * @throws RepositoryException if a repository error occurred
     */
    @Callable
    public Map<String, Object> setOrganisationChartRoot(String pageId) throws RepositoryException
    {
        Map<String, Object> result = new HashMap<>();
        
        Page page = _resolver.resolveById(pageId);
        
        if (_pageHandler.getOrganisationChartRootPages(page.getSiteName(), page.getSitemapName()) != null)
        {
            result.put("error", "already-exist");
            return result;
        }
        
        Map<String, Object> eventParams = new HashMap<>();
        eventParams.put(org.ametys.web.ObservationConstants.ARGS_PAGE, page);
        
        _addOrganisationChartRootProperty(page);
        
        // Live synchronization
        _notifyPageUpdated(page);
        
        // Indexation
        _observationManager.notify(new Event(ObservationConstants.EVENT_ORGANISATION_CHART_ROOT_UPDATED, _currentUserProvider.getUser(), eventParams));
        return result;
    }
    
    /**
     * Remove the organization chart root status to the given page
     * @param pageId The id of the page
     * @return A result map
     * @throws RepositoryException if a repository error occurred
     */
    @Callable
    public Map<String, Object> removeOrganisationChartRoot(String pageId) throws RepositoryException
    {
        Map<String, Object> result = new HashMap<>();
        
        Page page = _resolver.resolveById(pageId);
        
        if (page instanceof JCRAmetysObject)
        {
            if (!_pageHandler.isOrganisationChartRootPage((JCRAmetysObject) page))
            {
                result.put("error", "no-root");
                return result;
            }
            
            Map<String, Object> eventParams = new HashMap<>();
            eventParams.put(org.ametys.web.ObservationConstants.ARGS_PAGE, page);
            
            // Unindex pages for all workspaces before the properties were removed 
            _observationManager.notify(new Event(ObservationConstants.EVENT_ORGANISATION_CHART_ROOT_DELETING, _currentUserProvider.getUser(), eventParams));
            
            _removeUserDirectoryRootProperty(page);
            
            _notifyPageUpdated(page);
            
            // After live synchronization
            _observationManager.notify(new Event(ObservationConstants.EVENT_ORGANISATION_CHART_ROOT_DELETED, _currentUserProvider.getUser(), eventParams));
        }
        else
        {
            result.put("error", "no-root");
        }
        return result;
    }
    
    private void _addOrganisationChartRootProperty(Page page) throws RepositoryException
    {
        if (page instanceof JCRAmetysObject)
        {
            JCRAmetysObject jcrPage = (JCRAmetysObject) page;
            Node node = jcrPage.getNode();
            
            List<Value> values = new ArrayList<>();
            if (node.hasProperty(AmetysObjectResolver.VIRTUAL_PROPERTY))
            {
                values.addAll(Arrays.asList(node.getProperty(AmetysObjectResolver.VIRTUAL_PROPERTY).getValues()));
            }
            
            StringValue virtualUserDirectoryPageFactoryClassName = new StringValue(VirtualOrganisationChartPageFactory.class.getName());
            if (!values.contains(virtualUserDirectoryPageFactoryClassName))
            {
                values.add(virtualUserDirectoryPageFactoryClassName);
            }
            
            node.setProperty(AmetysObjectResolver.VIRTUAL_PROPERTY, values.toArray(new Value[values.size()]));
            jcrPage.saveChanges();
        }
    }
    
    private void _removeUserDirectoryRootProperty(Page page) throws RepositoryException
    {
        if (page instanceof JCRAmetysObject)
        {
            JCRAmetysObject jcrPage = (JCRAmetysObject) page;
            Node node = jcrPage.getNode();
            
            if (node.hasProperty(AmetysObjectResolver.VIRTUAL_PROPERTY))
            {
                List<Value> values = new ArrayList<>(Arrays.asList(node.getProperty(AmetysObjectResolver.VIRTUAL_PROPERTY).getValues()));
                int index = values.stream()
                        .map(LambdaUtils.wrap(Value::getString))
                        .collect(Collectors.toList())
                        .indexOf(VirtualOrganisationChartPageFactory.class.getName());
                
                if (index != -1)
                {
                    values.remove(index);
                    node.setProperty(AmetysObjectResolver.VIRTUAL_PROPERTY, values.toArray(new Value[values.size()]));
                }
                
                jcrPage.saveChanges();
            }
        }
    }
    
    private void _notifyPageUpdated(Page page)
    {
        Map<String, Object> eventParams = new HashMap<>();
        eventParams.put(org.ametys.web.ObservationConstants.ARGS_PAGE, page);
        _observationManager.notify(new Event(org.ametys.web.ObservationConstants.EVENT_PAGE_UPDATED, _currentUserProvider.getUser(), eventParams));
    }
}
