/*
 *  Copyright 2018 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.plugins.userdirectory.synchronize;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.slf4j.Logger;

import org.ametys.cms.repository.Content;
import org.ametys.core.user.UserIdentity;
import org.ametys.core.user.UserManager;
import org.ametys.plugins.contentio.synchronize.impl.LDAPSynchronizableContentsCollection;
import org.ametys.plugins.userdirectory.DeleteOrgUnitComponent;
import org.ametys.plugins.userdirectory.DeleteUserComponent;

/**
 * SQL synchronizable collection for users
 */
public class LDAPSynchronizableUsersCollection extends LDAPSynchronizableContentsCollection
{
    private static final String __PARAM_LDAP_TABLE_LOGIN = "login";
    private static final String __PARAM_POPULATION_ID = "populationId";
    
    /** The user manager */
    protected UserManager _userManager;

    /** The delete user component */
    protected DeleteUserComponent _deleteUserComponent;
    
    @Override
    public void service(ServiceManager smanager) throws ServiceException
    {
        super.service(smanager);
        _userManager = (UserManager) smanager.lookup(UserManager.ROLE);
        _deleteUserComponent = (DeleteUserComponent) smanager.lookup(DeleteUserComponent.ROLE);
    }
    
    public boolean handleRightAssignmentContext()
    {
        return false;
    }
    
    @Override
    public String getIdField()
    {
        return UserSCCConstants.USER_UNIQUE_ID_ATTRIBUTE_NAME;
    }
    
    @Override
    public Map<String, List<String>> getMapping()
    {
        Map<String, List<String>> mapping = super.getMapping();
        mapping.put(getIdField(), List.of(getLoginLDAPColumnName()));
        
        return mapping;
    }
    
    /**
     * Get the login column name of user LDAP
     * @return The login column name of user LDAP
     */
    public String getLoginLDAPColumnName()
    {
        return (String) getParameterValues().get(__PARAM_LDAP_TABLE_LOGIN);
    }
    
    /**
     * Get population Id
     * @return The population Id
     */
    public String getPopulationId()
    {
        return (String) getParameterValues().get(__PARAM_POPULATION_ID);
    }
    
    @Override
    protected Map<String, Map<String, Object>> internalSearch(Map<String, Object> searchParameters, int offset, int limit, List<Object> sort, Logger logger)
    {
        Map<String, Map<String, Object>> internalSearch = super.internalSearch(searchParameters, offset, limit, sort, logger);
        
        String population = getPopulationId();
        Map<String, Map<String, Object>> filteredSearchResult = new HashMap<>();
        for (String login : internalSearch.keySet())
        {
            if (_userManager.getUser(population, login) != null)
            {
                filteredSearchResult.put(login, internalSearch.get(login));
            }
            else
            {
                logger.warn("The user " + login + " don't belong to population " + population);
            }
        }
        
        return filteredSearchResult;
    }
    
    @Override
    protected Map<String, Object> getAdditionalAttributeValues(String idValue, Content content, Map<String, Object> additionalParameters, boolean create, Logger logger)
    {
        Map<String, Object> additionalRemoteValues = super.getAdditionalAttributeValues(idValue, content, additionalParameters, create, logger);
        UserIdentity user = new UserIdentity(idValue, getPopulationId());
        additionalRemoteValues.put(UserSCCConstants.USER_ATTRIBUTE_NAME, user);
        return additionalRemoteValues;
    }
    
    @Override
    protected int _deleteContents(List<Content> contentsToRemove, Logger logger)
    {
        return _deleteUserComponent.deleteContentsWithLog(contentsToRemove, Map.of(DeleteOrgUnitComponent.SCC_ID_PARAMETERS_KEY, getId()), Map.of(), logger);
    }
    
}
