/*
 *  Copyright 2022 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.plugins.userdirectory.transformation.xslt;

import java.io.IOException;
import java.io.InputStream;

import org.apache.avalon.framework.context.Context;
import org.apache.avalon.framework.context.ContextException;
import org.apache.avalon.framework.context.Contextualizable;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.cocoon.ProcessingException;
import org.apache.cocoon.components.ContextHelper;
import org.apache.cocoon.environment.Request;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.excalibur.source.Source;
import org.apache.excalibur.source.SourceResolver;
import org.slf4j.Logger;

import org.ametys.cms.data.ExplorerFile;
import org.ametys.cms.data.File;
import org.ametys.cms.repository.Content;
import org.ametys.cms.transformation.ImageResolverHelper;
import org.ametys.cms.transformation.xslt.ResolveURIComponent;
import org.ametys.core.user.UserIdentity;
import org.ametys.plugins.core.ui.user.ProfileImageProvider;
import org.ametys.plugins.core.ui.user.ProfileImageProvider.UserProfileImage;
import org.ametys.plugins.userdirectory.UserDirectoryHelper;
import org.ametys.runtime.plugin.component.LogEnabled;
import org.ametys.runtime.plugin.component.PluginAware;

/**
 * Helper to get the URI for the avatar of a user
 */
public class ResolveAvatarURIComponent implements Serviceable, Contextualizable, LogEnabled, PluginAware
{
    /** Attribute path for user's image */
    public static final String USER_CONTENT_IMAGE_PATH = "illustration/image";

    private static SourceResolver _srcResolver;
    private static UserDirectoryHelper _userDirectoryHelper;

    private static Context _context;

    private static Logger _logger;

    private static String _pluginName;

    private static ProfileImageProvider _profileImageProvider;

    public void service(ServiceManager manager) throws ServiceException
    {
        _profileImageProvider = (ProfileImageProvider) manager.lookup(ProfileImageProvider.ROLE);
        _srcResolver = (SourceResolver) manager.lookup(SourceResolver.ROLE);
        _userDirectoryHelper = (UserDirectoryHelper) manager.lookup(UserDirectoryHelper.ROLE);
    }
    
    public void setLogger(Logger logger)
    {
        _logger = logger;
    }
    
    public void setPluginInfo(String pluginName, String featureName, String id)
    {
        _pluginName = pluginName;
    }
    
    public void contextualize(Context context) throws ContextException
    {
        _context = context;
    }
    
    /**
     * Compute the avatar URI of a user
     * 
     * This method provide a convenient way to obtain the proxy URL on the base64 representation of the avatar
     * based on the request attribute "forceBase64Encoding".
     * 
     * @param login the login of the user
     * @param populationId the population of the user
     * @param lang the language used to retrieve the user content
     * @param size the size for the avatar
     * @return the avatar URI
     */
    public static String resolve(String login, String populationId, String lang, int size)
    {
        Request request = ContextHelper.getRequest(_context);
        boolean encodeBase64 = request.getAttribute("forceBase64Encoding") != null ? (Boolean) request.getAttribute("forceBase64Encoding") : false;
        if (!encodeBase64)
        {
            // Here we lost the lang information…
            // In case of empty login or populationID, we return an wrong URL here. It was the same when hardcoding the URL before
            return "_plugins/" + _pluginName + "/user/" + populationId + "/" + login + "/image_" + size;
        }
        else
        {
            return _getAvatarAsBase64(new UserIdentity(login, populationId), size, lang);
        }
    }

    private static String _getAvatarAsBase64(UserIdentity user, int size, String lang)
    {
        if (user == null)
        {
            Source source;
            try
            {
                source = _srcResolver.resolveURI("skin://resources/img/avatar.png");
                try (InputStream dataIs = source.getInputStream())
                {
                    return ImageResolverHelper.resolveImageAsBase64(dataIs, "image/png", size, size, 0, 0);
                }
            }
            catch (IOException e)
            {
                _logger.warn("Failed to retrieve generic avatar image from skin", e);
                return StringUtils.EMPTY;
            }
        }
        Content userContent = _userDirectoryHelper.getUserContent(user, lang);
        if (userContent != null && userContent.hasValue(USER_CONTENT_IMAGE_PATH))
        {
            File fileValue = userContent.getValue(USER_CONTENT_IMAGE_PATH);
            if (fileValue instanceof ExplorerFile)
            {
                // No need to set the request attribute for "forceBase64Encoding" as we have check earlier that it was set
                return ResolveURIComponent.resolveCroppedImage("explorer", ((ExplorerFile) fileValue).getResourceId(), size, size, false, false, true);
            }
            else
            {
                // Binary
                String imgUri = USER_CONTENT_IMAGE_PATH + "?contentId=" + userContent.getId();
                // No need to set the request attribute for "forceBase64Encoding" as we have check earlier that it was set
                return ResolveURIComponent.resolveCroppedImage("attribute", imgUri, size, size, false, false, true);
            }
        }
        else
        {
            try
            {
                UserProfileImage image = _profileImageProvider.getImage(user, null, size, 0);
                // same logic as in ProfileImageReader
                String filename = StringUtils.defaultIfEmpty(image.getFilename(), user.getLogin() + ".png");
                String format = FilenameUtils.getExtension(filename);
                
                try (InputStream dataIs = image.getInputstream())
                {
                    return ImageResolverHelper.resolveImageAsBase64(dataIs, "image/" + format, size, size, 0, 0);
                }
            }
            catch (ProcessingException | IOException e)
            {
                // TODO Auto-generated catch block
                _logger.warn("Failed to compute generic avatar for user {}", UserIdentity.userIdentityToString(user), e);
                return "";
            }
        }
    }
}
