/*
 *  Copyright 2017 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.plugins.userdirectory;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.net.ServerSocket;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.junit.Assert;
import org.junit.Test;

import org.ametys.cms.content.indexing.solr.observation.ObserverHelper;
import org.ametys.cms.data.ContentValue;
import org.ametys.cms.repository.Content;
import org.ametys.core.datasource.ConnectionHelper;
import org.ametys.core.right.Profile;
import org.ametys.core.right.ProfileAssignmentStorageExtensionPoint;
import org.ametys.core.right.RightProfilesDAO;
import org.ametys.core.script.SQLScriptHelper;
import org.ametys.core.util.MapUtils;
import org.ametys.plugins.contentio.synchronize.SynchronizableContentsCollection;
import org.ametys.plugins.contentio.synchronize.SynchronizableContentsCollectionDAO;
import org.ametys.plugins.contentio.synchronize.impl.DefaultSynchronizingContentOperator;
import org.ametys.plugins.repository.AmetysObjectResolver;
import org.ametys.plugins.repository.collection.AmetysObjectCollection;
import org.ametys.plugins.repository.data.holder.group.Repeater;
import org.ametys.plugins.repository.data.holder.group.RepeaterEntry;
import org.ametys.plugins.repository.data.holder.group.ModelAwareRepeater;
import org.ametys.plugins.repository.data.holder.group.ModelAwareRepeaterEntry;
import org.ametys.plugins.repositoryapp.RepositoryProvider;
import org.ametys.plugins.userdirectory.synchronize.SQLSynchronizableUDOrgunitCollection;
import org.ametys.runtime.test.framework.AbstractPopulationAwareTestCase;
import org.ametys.runtime.test.framework.CocoonTestCase.AllFeatures;
import org.ametys.runtime.test.framework.TestConfigSet;
import org.ametys.runtime.test.framework.Unoptimized;

/**
 * Test cases for OrgUnit SCC from User directory.
 */
@Unoptimized
@AllFeatures
public class SCCOrgUnitTestCase extends AbstractPopulationAwareTestCase
{
    /** The SCC DAO */
    protected static SynchronizableContentsCollectionDAO _sccDAO;
    
    /** The Organisation Chart page hanlder */
    protected static OrganisationChartPageHandler _orgChartPageHandler;

    /** The resolver for Ametys Objects */
    protected static AmetysObjectResolver _resolver;

    /** The repository provider */
    protected static RepositoryProvider _repositoryProvider;
    
    /** The rights profiles DAO **/
    protected static RightProfilesDAO _profilesDAO;
    
    /** The profile assignment storage Extension point **/
    protected static ProfileAssignmentStorageExtensionPoint _profileAssignmentStorageEP;
    
    /** The user directory helper */
    protected UserDirectoryHelper _userDirectoryHelper;
    
    @Override
    public String getDBType()
    {
        return ConnectionHelper.DATABASE_MYSQL;
    }
    
    @Override
    public TestConfigSet getConfigSet()
    {
        return super.getConfigSet()
                .withRuntimeFile("test/environments/runtime.xml")
                .withConfigFile("test/environments/config.xml")
                .withWebappPath("test/environments/webapp");
    }
    
    @Override
    public Map<String, String> variableValues()
    {
        // FIXME do it just for valid Solr URL on a free port (not used), it does not matter that it will not succeed to reach it
        int solrPort = _getFreePort();
        
        return MapUtils.merge(
                super.variableValues(), 
                Map.of(
                        "solrPort", String.valueOf(solrPort)
                )
        );
    }
    
    private static int _getFreePort()
    {
        try (ServerSocket s = new ServerSocket(0);)
        {
            return s.getLocalPort();
        }
        catch (IOException e)
        {
            throw new RuntimeException("Unable to find a free port", e);
        }
    }
    
    @Override
    protected void essentialsBefore() throws Exception
    {
        ObserverHelper.suspendObservationForIndexation();
        super.essentialsBefore();
    }
    
    @Override
    public void lookupComponents(ServiceManager manager) throws ServiceException
    {
        super.lookupComponents(manager);
        
        _sccDAO = (SynchronizableContentsCollectionDAO) manager.lookup(SynchronizableContentsCollectionDAO.ROLE);
        _orgChartPageHandler = (OrganisationChartPageHandler) manager.lookup(OrganisationChartPageHandler.ROLE);
        _resolver = (AmetysObjectResolver) manager.lookup(AmetysObjectResolver.ROLE);
        _repositoryProvider = (RepositoryProvider) manager.lookup(RepositoryProvider.ROLE);
        _profilesDAO = (RightProfilesDAO) manager.lookup(RightProfilesDAO.ROLE);
        _profileAssignmentStorageEP = (ProfileAssignmentStorageExtensionPoint) manager.lookup(ProfileAssignmentStorageExtensionPoint.ROLE);
        _userDirectoryHelper = (UserDirectoryHelper) manager.lookup(UserDirectoryHelper.ROLE);
    }
    
    @Override
    public void prepareComponents() throws Exception
    {
        super.prepareComponents();
        
        Profile profile = _profilesDAO.addProfile("myProfile");
        _profilesDAO.addRight(profile, "Workflow_Rights_Edition_Online");
        
        AmetysObjectCollection rootOfContents = _userDirectoryHelper.getUserDirectoryRootContent(true);
        _profileAssignmentStorageEP.allowProfileToUser(CURRENT_USER_TEST_IDENTITY, profile.getId(), rootOfContents);
    }
    
    @Override
    public File[] getStartScriptFiles()
    {
        String dbType = getDBType();
        return new File[] {
            new File("test/environments/scripts/jdbc/" + dbType + "/deleteTables.sql"),
            new File("test/environments/scripts/jdbc/" + dbType + "/initTables.sql")
        };
    }
    
//    /**
//     * Before each test, init application state
//     * @throws Exception If an error occurs.
//     */
//    @Before
//    public void sccBeforeTest() throws Exception
//    {
//        _initSqlTables();
//    }
//    
//    private void _initSqlTables() throws Exception
//    {
//        String dbType = getDBType();
//        List<File> scripts = Arrays.asList(
//                new File("test/environments/scripts/jdbc/" + dbType + "/deleteTables.sql"),
//                new File("test/environments/scripts/jdbc/" + dbType + "/initTables.sql"));
//        ScriptHelper.executeStartScripts(scripts, Collections.emptyList(), SQLDataSourceManager.getStaticFileConfiguration().getPath(), dbType);
//    }
    
    // TODO The following does not work due to a weird JCR node exception, so the test is annotated @Unoptimized as long as it is not fixed
//    /**
//     * After each test, clean application state
//     * @throws Exception If an error occurs.
//     */
//    @After
//    public void sccAfterTest() throws Exception
//    {
//        _removeScc();
//        _removeContents();
//    }
//    
//    private void _removeScc()
//    {
//        List<SynchronizableContentsCollection> collections = _sccDAO.getSynchronizableContentsCollections();
//        for (SynchronizableContentsCollection collection : collections)
//        {
//            String id = collection.getId();
//            try
//            {
//                _sccDAO.removeCollection(id);
//                _logger.info("Collection '{}' was successfully removed.", id);
//            }
//            catch (Exception e)
//            {
//                _logger.error("An error occurred when trying to remove collection '{}'", id, e);
//            }
//        }
//    }
//    
//    private void _removeContents() throws RepositoryException
//    {
//        _removeContentsInWorkspace(WebConstants.LIVE_WORKSPACE);
//        _removeContentsInWorkspace(RepositoryConstants.DEFAULT_WORKSPACE);
//    }
//    
//    private void _removeContentsInWorkspace(String workspaceName) throws RepositoryException
//    {
////        Session session = _repositoryProvider.getSession(workspaceName);
//        Session session = _repositoryProvider.getRepository().login(workspaceName);
//        
//        String query = ContentQueryHelper.getContentXPathQuery(null);
//        List<AmetysObject> contents = _resolver.query(query, session)
//                .stream()
//                .collect(Collectors.toList());
//        for (AmetysObject content : contents)
//        {
//            if (content instanceof Content && content instanceof JCRAmetysObject)
//            {
//                JCRAmetysObject jcrContent = (JCRAmetysObject) content;
//                String id = jcrContent.getId();
//                String name = jcrContent.getName();
//                jcrContent.getNode().remove();
//                _logger.info("Content '{}' ({}) was removed in workspace {}.", id, name, workspaceName);
//            }
//        }
//        if (session.hasPendingChanges())
//        {
//            session.save();
//        }
//        
//        long remainingContents = _resolver.query(query).getSize();
//        _logger.info("{} contents remain after removing in workspace {}.", remainingContents, workspaceName);
//    }
    
    /**
     * Test that at startup, the number of SCC is the expected number.
     * @throws Exception if an error occurs
     */
    @Test
    public void testOrgUnitSynchroWithUser() throws Exception
    {
        /* Count available languages */
        List<String> languages = _getLanguages();
        int nbLangs = languages.size();

        /* Create the User SCC */
        Map<String, Object> sccUserParameters = new HashMap<>();
        sccUserParameters.put("label", "Collection user TEST");
        sccUserParameters.put("contentType", "agent");
        sccUserParameters.put("contentPrefix", "test-");
        sccUserParameters.put("workflowName", "user");
        sccUserParameters.put("initialActionId", 11);
        sccUserParameters.put("synchronizeActionId", 800);
        sccUserParameters.put("validateActionId", 41);
        sccUserParameters.put("contentOperator", DefaultSynchronizingContentOperator.class.getName());
        sccUserParameters.put("modelId", "org.ametys.plugins.contentio.synchronize.collection.sql");
        sccUserParameters.put("languages", languages);
        sccUserParameters.put("org.ametys.plugins.contentio.synchronize.collection.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "datasourceId", "SQL-test");
        sccUserParameters.put("org.ametys.plugins.contentio.synchronize.collection.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "tableName", "synchro_sql_users");
        sccUserParameters.put("org.ametys.plugins.contentio.synchronize.collection.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "idField", "login");
        sccUserParameters.put("org.ametys.plugins.contentio.synchronize.collection.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "mapping", "["
            + "{\"metadata-ref\":\"title\",\"attribute\":\"login\",\"synchro\":false},"
            + "{\"metadata-ref\":\"login\",\"attribute\":\"login\",\"synchro\":false},"
            + "{\"metadata-ref\":\"firstname\",\"attribute\":\"prenom\",\"synchro\":false},"
            + "{\"metadata-ref\":\"lastname\",\"attribute\":\"nom\",\"synchro\":false}]");
        
        String userCollectionId = _sccDAO.addCollection(sccUserParameters);
        Assert.assertNotNull(userCollectionId);
        
        SynchronizableContentsCollection userScc = _sccDAO.getSynchronizableContentsCollection(userCollectionId);
        Assert.assertNotNull(userScc);

        /* Populate with the User SCC */
        List<? extends Content> userContents = userScc.populate(_logger);
        Assert.assertNotNull(userContents);
        Assert.assertEquals(nbLangs * 10, userContents.size());
        
        /* Create the SCC */
        Map<String, Object> sccParameters = new HashMap<>();
        sccParameters.put("label", "Collection TEST");
        sccParameters.put("contentType", "org.ametys.plugins.userdirectory.Content.udorgunit");
        sccParameters.put("contentPrefix", "test-");
        sccParameters.put("workflowName", "udorgunit");
        sccParameters.put("initialActionId", 11);
        sccParameters.put("synchronizeActionId", 800);
        sccParameters.put("validateActionId", 41);
        sccParameters.put("contentOperator", DefaultSynchronizingContentOperator.class.getName());
        sccParameters.put("modelId", "org.ametys.plugins.userdirectory.synchronize.orgunit.sql");
        sccParameters.put("removalSync", true);
        sccParameters.put("ignoreRestrictions", true);
        sccParameters.put("languages", languages);
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "datasourceId", "SQL-test");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "tableName", "synchro_sql_orgunits");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "idField", "orgUnitId");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "orgunitRemoteIdColumnName", "id");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "tableNameUser", "synchro_sql_users");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "orgUnitJoinColumnName", "orgunitId");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "loginUser", "login");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "loginColumnName", "login");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "roleColumnName", "role");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "mapping", "["
            + "{\"metadata-ref\":\"title\",\"attribute\":\"title\",\"synchro\":false},"
            + "{\"metadata-ref\":\"orgUnitId\",\"attribute\":\"id\",\"synchro\":false},"
            + "{\"metadata-ref\":\"parentOrgUnit\",\"attribute\":\"parentId\",\"synchro\":false},"
            + "{\"metadata-ref\":\"abstract\",\"attribute\":\"abstract\",\"synchro\":false}]");
        
        String collectionId = _sccDAO.addCollection(sccParameters);
        Assert.assertNotNull(collectionId);
        
        SynchronizableContentsCollection scc = _sccDAO.getSynchronizableContentsCollection(collectionId);
        Assert.assertNotNull(scc);

        /* Populate with the SCC */
        List<? extends Content> contents = scc.populate(_logger);
        Assert.assertNotNull(contents);
        
        Map<String, Integer> result = scc.getSynchronizationResult();
        Assert.assertEquals(Integer.valueOf(nbLangs * 5), result.get(SynchronizableContentsCollection.RESULT_NB_CREATED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_SYNCHRONIZED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_NOT_CHANGED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_DELETED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(nbLangs * 5), result.get(SQLSynchronizableUDOrgunitCollection.RESULT_NB_SYNCHRONIZED_ORGUNITS_RELATIONS));
        
        _testOrgUnits(contents, nbLangs);
        _testUsersOnOrgUnits(contents, nbLangs);
        
        List<File> files = new ArrayList<>();
        files.add(new File("test/environments/scripts/jdbc/" + getDBType() + "/deleteTables.sql"));
        files.add(new File("test/environments/scripts/jdbc/" + getDBType() + "/initNewTables.sql"));
        _updateScript(files);
        
        /* Populate with the SCC */
        contents = scc.populate(_logger);
        Assert.assertNotNull(contents);
        
        result = scc.getSynchronizationResult();
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_CREATED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(nbLangs * 3), result.get(SynchronizableContentsCollection.RESULT_NB_SYNCHRONIZED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(nbLangs), result.get(SynchronizableContentsCollection.RESULT_NB_NOT_CHANGED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(nbLangs), result.get(SynchronizableContentsCollection.RESULT_NB_DELETED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(nbLangs * 2), result.get(SQLSynchronizableUDOrgunitCollection.RESULT_NB_SYNCHRONIZED_ORGUNITS_RELATIONS));
        
        _testNewOrgUnits(contents, nbLangs);
        _testNewUsersOnOrgUnits(contents, nbLangs);
    }
    
    /**
     * Test that at startup, the number of SCC is the expected number.
     * @throws Exception if an error occurs
     */
    @Test
    public void testOrgUnitSynchroWithNoUser() throws Exception
    {
        /* Count available languages */
        List<String> languages = _getLanguages();
        int nbLangs = languages.size();

        /* Create the SCC */
        Map<String, Object> sccParameters = new HashMap<>();
        sccParameters.put("label", "Collection no user TEST");
        sccParameters.put("contentType", "org.ametys.plugins.userdirectory.Content.udorgunit");
        sccParameters.put("contentPrefix", "test-");
        sccParameters.put("workflowName", "udorgunit");
        sccParameters.put("initialActionId", 11);
        sccParameters.put("synchronizeActionId", 800);
        sccParameters.put("validateActionId", 41);
        sccParameters.put("contentOperator", DefaultSynchronizingContentOperator.class.getName());
        sccParameters.put("modelId", "org.ametys.plugins.userdirectory.synchronize.orgunit.sql");
        sccParameters.put("languages", languages);
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "datasourceId", "SQL-test");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "tableName", "synchro_sql_orgunits");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "idField", "orgUnitId");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "orgunitRemoteIdColumnName", "id");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "tableNameUser", "synchro_sql_users");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "orgUnitJoinColumnName", "orgunitId");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "loginUser", "login");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "loginColumnName", "login");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "roleColumnName", "role");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "mapping", "["
            + "{\"metadata-ref\":\"title\",\"attribute\":\"title\",\"synchro\":false},"
            + "{\"metadata-ref\":\"orgUnitId\",\"attribute\":\"id\",\"synchro\":false},"
            + "{\"metadata-ref\":\"parentOrgUnit\",\"attribute\":\"parentId\",\"synchro\":false},"
            + "{\"metadata-ref\":\"abstract\",\"attribute\":\"abstract\",\"synchro\":false}]");
        
        String collectionId = _sccDAO.addCollection(sccParameters);
        Assert.assertNotNull(collectionId);
        
        SynchronizableContentsCollection scc = _sccDAO.getSynchronizableContentsCollection(collectionId);
        Assert.assertNotNull(scc);

        /* Populate with the SCC */
        List<? extends Content> contents = scc.populate(_logger);
        Assert.assertNotNull(contents);
        
        Map<String, Integer> result = scc.getSynchronizationResult();
        Assert.assertEquals(Integer.valueOf(nbLangs * 5), result.get(SynchronizableContentsCollection.RESULT_NB_CREATED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_SYNCHRONIZED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_NOT_CHANGED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_DELETED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(nbLangs * 5), result.get(SQLSynchronizableUDOrgunitCollection.RESULT_NB_SYNCHRONIZED_ORGUNITS_RELATIONS));
        
        _testOrgUnits(contents, nbLangs);
    }
    
    /**
     * Test that at startup, the number of SCC is the expected number.
     * @throws Exception if an error occurs
     */
    @Test
    public void testOrgUnitSynchroWithNoChange() throws Exception
    {
        /* Count available languages */
        List<String> languages = _getLanguages();
        int nbLangs = languages.size();

        /* Create the SCC */
        Map<String, Object> sccParameters = new HashMap<>();
        sccParameters.put("label", "Collection no change TEST");
        sccParameters.put("contentType", "org.ametys.plugins.userdirectory.Content.udorgunit");
        sccParameters.put("contentPrefix", "test-");
        sccParameters.put("workflowName", "udorgunit");
        sccParameters.put("initialActionId", 11);
        sccParameters.put("synchronizeActionId", 800);
        sccParameters.put("validateActionId", 41);
        sccParameters.put("contentOperator", DefaultSynchronizingContentOperator.class.getName());
        sccParameters.put("modelId", "org.ametys.plugins.userdirectory.synchronize.orgunit.sql");
        sccParameters.put("languages", languages);
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "datasourceId", "SQL-test");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "tableName", "synchro_sql_orgunits");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "idField", "orgUnitId");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "orgunitRemoteIdColumnName", "id");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "tableNameUser", "synchro_sql_users");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "orgUnitJoinColumnName", "orgunitId");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "loginUser", "login");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "loginColumnName", "login");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "roleColumnName", "role");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "mapping", "["
            + "{\"metadata-ref\":\"title\",\"attribute\":\"title\",\"synchro\":false},"
            + "{\"metadata-ref\":\"orgUnitId\",\"attribute\":\"id\",\"synchro\":false},"
            + "{\"metadata-ref\":\"parentOrgUnit\",\"attribute\":\"parentId\",\"synchro\":false},"
            + "{\"metadata-ref\":\"abstract\",\"attribute\":\"abstract\",\"synchro\":false}]");
        
        String collectionId = _sccDAO.addCollection(sccParameters);
        Assert.assertNotNull(collectionId);
        
        SynchronizableContentsCollection scc = _sccDAO.getSynchronizableContentsCollection(collectionId);
        Assert.assertNotNull(scc);

        /* Populate with the SCC */
        List<? extends Content> contents = scc.populate(_logger);
        Assert.assertNotNull(contents);
        
        Map<String, Integer> result = scc.getSynchronizationResult();
        Assert.assertEquals(Integer.valueOf(nbLangs * 5), result.get(SynchronizableContentsCollection.RESULT_NB_CREATED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_SYNCHRONIZED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_NOT_CHANGED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_DELETED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(nbLangs * 5), result.get(SQLSynchronizableUDOrgunitCollection.RESULT_NB_SYNCHRONIZED_ORGUNITS_RELATIONS));
        
        _testOrgUnits(contents, nbLangs);
        
        /* Populate with the SCC using same script, the data should not change */
        contents = scc.populate(_logger);
        Assert.assertNotNull(contents);
        
        result = scc.getSynchronizationResult();
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_CREATED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_SYNCHRONIZED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(nbLangs * 5), result.get(SynchronizableContentsCollection.RESULT_NB_NOT_CHANGED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_DELETED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SQLSynchronizableUDOrgunitCollection.RESULT_NB_SYNCHRONIZED_ORGUNITS_RELATIONS));
        
        _testOrgUnits(contents, nbLangs);
    }
    
    /**
     * Test that at startup, the number of SCC is the expected number.
     * @throws Exception if an error occurs
     */
    @Test
    public void testOrgUnitSynchroWithNoRole() throws Exception
    {
        /* Count available languages */
        List<String> languages = _getLanguages();
        int nbLangs = languages.size();

        /* Create the User SCC */
        Map<String, Object> sccUserParameters = new HashMap<>();
        sccUserParameters.put("label", "Collection no role user TEST");
        sccUserParameters.put("contentType", "agent");
        sccUserParameters.put("contentPrefix", "test-");
        sccUserParameters.put("workflowName", "user");
        sccUserParameters.put("initialActionId", 11);
        sccUserParameters.put("synchronizeActionId", 800);
        sccUserParameters.put("validateActionId", 41);
        sccUserParameters.put("contentOperator", DefaultSynchronizingContentOperator.class.getName());
        sccUserParameters.put("languages", languages);
        sccUserParameters.put("modelId", "org.ametys.plugins.contentio.synchronize.collection.sql");
        sccUserParameters.put("org.ametys.plugins.contentio.synchronize.collection.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "datasourceId", "SQL-test");
        sccUserParameters.put("org.ametys.plugins.contentio.synchronize.collection.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "tableName", "synchro_sql_users");
        sccUserParameters.put("org.ametys.plugins.contentio.synchronize.collection.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "idField", "login");
        sccUserParameters.put("org.ametys.plugins.contentio.synchronize.collection.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "mapping", "["
            + "{\"metadata-ref\":\"title\",\"attribute\":\"login\",\"synchro\":false},"
            + "{\"metadata-ref\":\"login\",\"attribute\":\"login\",\"synchro\":false},"
            + "{\"metadata-ref\":\"firstname\",\"attribute\":\"prenom\",\"synchro\":false},"
            + "{\"metadata-ref\":\"lastname\",\"attribute\":\"nom\",\"synchro\":false}]");
        
        String userCollectionId = _sccDAO.addCollection(sccUserParameters);
        Assert.assertNotNull(userCollectionId);
        
        SynchronizableContentsCollection userScc = _sccDAO.getSynchronizableContentsCollection(userCollectionId);
        Assert.assertNotNull(userScc);

        /* Populate with the User SCC */
        List<? extends Content> userContents = userScc.populate(_logger);
        Assert.assertNotNull(userContents);
        Assert.assertEquals(nbLangs * 10, userContents.size());
        
        /* Create the SCC */
        Map<String, Object> sccParameters = new HashMap<>();
        sccParameters.put("label", "Collection no role TEST");
        sccParameters.put("contentType", "org.ametys.plugins.userdirectory.Content.udorgunit");
        sccParameters.put("contentPrefix", "test-");
        sccParameters.put("workflowName", "udorgunit");
        sccParameters.put("initialActionId", 11);
        sccParameters.put("synchronizeActionId", 800);
        sccParameters.put("validateActionId", 41);
        sccParameters.put("contentOperator", DefaultSynchronizingContentOperator.class.getName());
        sccParameters.put("languages", languages);
        sccParameters.put("modelId", "org.ametys.plugins.userdirectory.synchronize.orgunit.sql");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "datasourceId", "SQL-test");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "tableName", "synchro_sql_orgunits");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "idField", "orgUnitId");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "orgunitRemoteIdColumnName", "id");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "tableNameUser", "synchro_sql_users");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "orgUnitJoinColumnName", "orgunitId");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "loginUser", "login");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "loginColumnName", "login");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "mapping", "["
            + "{\"metadata-ref\":\"title\",\"attribute\":\"title\",\"synchro\":false},"
            + "{\"metadata-ref\":\"orgUnitId\",\"attribute\":\"id\",\"synchro\":false},"
            + "{\"metadata-ref\":\"parentOrgUnit\",\"attribute\":\"parentId\",\"synchro\":false},"
            + "{\"metadata-ref\":\"abstract\",\"attribute\":\"abstract\",\"synchro\":false}]");
        
        String collectionId = _sccDAO.addCollection(sccParameters);
        Assert.assertNotNull(collectionId);
        
        SynchronizableContentsCollection scc = _sccDAO.getSynchronizableContentsCollection(collectionId);
        Assert.assertNotNull(scc);

        /* Populate with the SCC */
        List<? extends Content> contents = scc.populate(_logger);
        Assert.assertNotNull(contents);
        
        Map<String, Integer> result = scc.getSynchronizationResult();
        Assert.assertEquals(Integer.valueOf(nbLangs * 5), result.get(SynchronizableContentsCollection.RESULT_NB_CREATED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_SYNCHRONIZED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_NOT_CHANGED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_DELETED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(nbLangs * 5), result.get(SQLSynchronizableUDOrgunitCollection.RESULT_NB_SYNCHRONIZED_ORGUNITS_RELATIONS));
        
        _testOrgUnits(contents, nbLangs);
        _testUsersWithNoRoleOnOrgUnits(contents, nbLangs);
    }
    
    /**
     * Test that at startup, the number of SCC is the expected number.
     * @throws Exception if an error occurs
     */
    @Test
    public void testOrgUnitSynchroWithNoParentOrgunit() throws Exception
    {
        /* Count available languages */
        List<String> languages = _getLanguages();
        int nbLangs = languages.size();

        /* Create the SCC */
        Map<String, Object> sccParameters = new HashMap<>();
        sccParameters.put("label", "Collection no parent TEST");
        sccParameters.put("contentType", "org.ametys.plugins.userdirectory.Content.udorgunit");
        sccParameters.put("contentPrefix", "test-");
        sccParameters.put("workflowName", "udorgunit");
        sccParameters.put("initialActionId", 11);
        sccParameters.put("synchronizeActionId", 800);
        sccParameters.put("validateActionId", 41);
        sccParameters.put("contentOperator", DefaultSynchronizingContentOperator.class.getName());
        sccParameters.put("languages", languages);
        sccParameters.put("modelId", "org.ametys.plugins.userdirectory.synchronize.orgunit.sql");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "datasourceId", "SQL-test");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "tableName", "synchro_sql_orgunits");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "idField", "orgUnitId");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "orgunitRemoteIdColumnName", "id");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "tableNameUser", "synchro_sql_users");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "orgUnitJoinColumnName", "orgunitId");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "loginUser", "login");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "loginColumnName", "login");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "roleColumnName", "role");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.orgunit.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "mapping", "["
            + "{\"metadata-ref\":\"title\",\"attribute\":\"title\",\"synchro\":false},"
            + "{\"metadata-ref\":\"orgUnitId\",\"attribute\":\"id\",\"synchro\":false},"
            + "{\"metadata-ref\":\"abstract\",\"attribute\":\"abstract\",\"synchro\":false}]");
        
        String collectionId = _sccDAO.addCollection(sccParameters);
        Assert.assertNotNull(collectionId);
        
        SynchronizableContentsCollection scc = _sccDAO.getSynchronizableContentsCollection(collectionId);
        Assert.assertNotNull(scc);

        /* Populate with the SCC */
        List<? extends Content> contents = scc.populate(_logger);
        Assert.assertNotNull(contents);
        
        Map<String, Integer> result = scc.getSynchronizationResult();
        Assert.assertEquals(Integer.valueOf(nbLangs * 5), result.get(SynchronizableContentsCollection.RESULT_NB_CREATED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_SYNCHRONIZED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_NOT_CHANGED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_DELETED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(nbLangs * 5), result.get(SQLSynchronizableUDOrgunitCollection.RESULT_NB_SYNCHRONIZED_ORGUNITS_RELATIONS));
        
        _testOrgUnitsWithNoParent(contents, nbLangs);
    }

    /**
     * Test the org units
     * @param orgUnits the synchronized org units
     * @param nbLangs the number of languages
     */
    protected void _testOrgUnits(List<? extends Content> orgUnits, int nbLangs)
    {
        Assert.assertEquals(5 * nbLangs, orgUnits.size());
        
        for (int i = 0; i < nbLangs; i++)
        {
            int offset = 0;
            Content ametys = orgUnits.get(offset++ * nbLangs + i);
            Content dev = orgUnits.get(offset++ * nbLangs + i);
            Content com = orgUnits.get(offset++ * nbLangs + i);
            Content projects = orgUnits.get(offset++ * nbLangs + i);
            Content games = orgUnits.get(offset++ * nbLangs + i);

            // Ametys
            Assert.assertEquals("1", ametys.getInternalDataHolder().getValue("orgunit-remote-id"));
            Assert.assertEquals("1", ametys.getValue("orgUnitId"));
            Assert.assertEquals("Pôle Ametys", ametys.getValue("title"));
            Assert.assertEquals("Pôle Ametys", ametys.getValue("abstract"));
            Content parent = _orgChartPageHandler.getParentContent(ametys);
            Assert.assertNull(parent);
            List<Content> childContents = _orgChartPageHandler.getChildContents(ametys);
            Assert.assertEquals(3, childContents.size());
            Assert.assertTrue(childContents.contains(dev));
            Assert.assertTrue(childContents.contains(com));
            Assert.assertTrue(childContents.contains(projects));

            // Dev
            Assert.assertEquals("2", dev.getInternalDataHolder().getValue("orgunit-remote-id"));
            Assert.assertEquals("2", dev.getValue("orgUnitId"));
            Assert.assertEquals("Service développement", dev.getValue("title"));
            Assert.assertEquals("Service développement", dev.getValue("abstract"));
            parent = _orgChartPageHandler.getParentContent(dev);
            Assert.assertEquals(ametys, parent);
            childContents = _orgChartPageHandler.getChildContents(dev);
            Assert.assertEquals(1, childContents.size());
            Assert.assertTrue(childContents.contains(games));

            // Com
            Assert.assertEquals("3", com.getInternalDataHolder().getValue("orgunit-remote-id"));
            Assert.assertEquals("3", com.getValue("orgUnitId"));
            Assert.assertEquals("Service commercial", com.getValue("title"));
            Assert.assertEquals("Service commercial", com.getValue("abstract"));
            parent = _orgChartPageHandler.getParentContent(com);
            Assert.assertEquals(ametys, parent);
            childContents = _orgChartPageHandler.getChildContents(com);
            Assert.assertTrue(childContents.isEmpty());

            // Projects
            Assert.assertEquals("4", projects.getInternalDataHolder().getValue("orgunit-remote-id"));
            Assert.assertEquals("4", projects.getValue("orgUnitId"));
            Assert.assertEquals("Service des projets", projects.getValue("title"));
            Assert.assertEquals("Service des projets", projects.getValue("abstract"));
            parent = _orgChartPageHandler.getParentContent(projects);
            Assert.assertEquals(ametys, parent);
            childContents = _orgChartPageHandler.getChildContents(projects);
            Assert.assertTrue(childContents.isEmpty());

            // Games
            Assert.assertEquals("5", games.getInternalDataHolder().getValue("orgunit-remote-id"));
            Assert.assertEquals("5", games.getValue("orgUnitId"));
            Assert.assertEquals("Salle de jeux", games.getValue("title"));
            Assert.assertEquals("Salle de jeux", games.getValue("abstract"));
            parent = _orgChartPageHandler.getParentContent(games);
            Assert.assertEquals(dev, parent);
            childContents = _orgChartPageHandler.getChildContents(games);
            Assert.assertTrue(childContents.isEmpty());
        }
    }
    
    /**
     * Test the org units with no parent
     * @param orgUnits the synchronized org units
     * @param nbLangs the number of languages
     */
    protected void _testOrgUnitsWithNoParent(List<? extends Content> orgUnits, int nbLangs)
    {
        Assert.assertEquals(5 * nbLangs, orgUnits.size());
        
        for (int i = 0; i < nbLangs; i++)
        {
            int offset = 0;
            Content ametys = orgUnits.get(offset++ * nbLangs + i);
            Content dev = orgUnits.get(offset++ * nbLangs + i);
            Content com = orgUnits.get(offset++ * nbLangs + i);
            Content projects = orgUnits.get(offset++ * nbLangs + i);
            Content games = orgUnits.get(offset++ * nbLangs + i);
            
            // Ametys
            Assert.assertEquals("1", ametys.getInternalDataHolder().getValue("orgunit-remote-id"));
            Assert.assertEquals("1", ametys.getValue("orgUnitId"));
            Assert.assertEquals("Pôle Ametys", ametys.getValue("title"));
            Assert.assertEquals("Pôle Ametys", ametys.getValue("abstract"));
            
            // Dev
            Assert.assertEquals("2", dev.getInternalDataHolder().getValue("orgunit-remote-id"));
            Assert.assertEquals("2", dev.getValue("orgUnitId"));
            Assert.assertEquals("Service développement", dev.getValue("title"));
            Assert.assertEquals("Service développement", dev.getValue("abstract"));
            
            // Com
            Assert.assertEquals("3", com.getInternalDataHolder().getValue("orgunit-remote-id"));
            Assert.assertEquals("3", com.getValue("orgUnitId"));
            Assert.assertEquals("Service commercial", com.getValue("title"));
            Assert.assertEquals("Service commercial", com.getValue("abstract"));
            
            // Projects
            Assert.assertEquals("4", projects.getInternalDataHolder().getValue("orgunit-remote-id"));
            Assert.assertEquals("4", projects.getValue("orgUnitId"));
            Assert.assertEquals("Service des projets", projects.getValue("title"));
            Assert.assertEquals("Service des projets", projects.getValue("abstract"));
            
            // Games
            Assert.assertEquals("5", games.getInternalDataHolder().getValue("orgunit-remote-id"));
            Assert.assertEquals("5", games.getValue("orgUnitId"));
            Assert.assertEquals("Salle de jeux", games.getValue("title"));
            Assert.assertEquals("Salle de jeux", games.getValue("abstract"));
        }
        
        // Check that there is no parent on each org unit
        for (Content orgUnit : orgUnits)
        {
            Content parent = _orgChartPageHandler.getParentContent(orgUnit);
            Assert.assertNull(parent);
            List<Content> childContents = _orgChartPageHandler.getChildContents(orgUnit);
            Assert.assertTrue(childContents.isEmpty());
        }
    }
    
    /**
     * Test the users on org units
     * @param orgUnits the synchronized org units
     * @param nbLangs the number of languages
     */
    protected void _testUsersOnOrgUnits(List<? extends Content> orgUnits, int nbLangs)
    {
        for (int i = 0; i < nbLangs; i++)
        {
            int offset = 0;
            Content ametys = orgUnits.get(offset++ * nbLangs + i);
            Content dev = orgUnits.get(offset++ * nbLangs + i);
            Content com = orgUnits.get(offset++ * nbLangs + i);
            Content projects = orgUnits.get(offset++ * nbLangs + i);
            Content games = orgUnits.get(offset++ * nbLangs + i);

            // Ametys
            ModelAwareRepeater ametysUsers = ametys.getRepeater("users");
            Assert.assertEquals(2, ametysUsers.getSize());
            ModelAwareRepeaterEntry ametysUser1 = ametysUsers.getEntry(1);
            Assert.assertEquals("cdamioli", ((ContentValue) ametysUser1.getValue("user")).getValue("login"));
            Assert.assertEquals("President", ametysUser1.getValue("role"));
            ModelAwareRepeaterEntry ametysUser2 = ametysUsers.getEntry(2);
            Assert.assertEquals("gubersfeld", ((ContentValue) ametysUser2.getValue("user")).getValue("login"));
            Assert.assertEquals("Directeur commercial", ametysUser2.getValue("role"));

            // Dev
            ModelAwareRepeater devUsers = dev.getRepeater("users");
            Assert.assertEquals(4, devUsers.getSize());
            ModelAwareRepeaterEntry devUser1 = devUsers.getEntry(1);
            Assert.assertEquals("lbouchet", ((ContentValue) devUser1.getValue("user")).getValue("login"));
            Assert.assertFalse(devUser1.hasValue("role"));
            ModelAwareRepeaterEntry devUser2 = devUsers.getEntry(2);
            Assert.assertEquals("sprieul", ((ContentValue) devUser2.getValue("user")).getValue("login"));
            Assert.assertFalse(devUser2.hasValue("role"));
            ModelAwareRepeaterEntry devUser3 = devUsers.getEntry(3);
            Assert.assertEquals("laumeunier", ((ContentValue) devUser3.getValue("user")).getValue("login"));
            Assert.assertEquals("Chef du developpement", devUser3.getValue("role"));
            ModelAwareRepeaterEntry devUser4 = devUsers.getEntry(4);
            Assert.assertEquals("glefebvre", ((ContentValue) devUser4.getValue("user")).getValue("login"));
            Assert.assertFalse(devUser4.hasValue("role"));

            // Com
            ModelAwareRepeater comUsers = com.getRepeater("users");
            Assert.assertEquals(2, comUsers.getSize());
            ModelAwareRepeaterEntry comUser1 = comUsers.getEntry(1);
            Assert.assertEquals("jbourak", ((ContentValue) comUser1.getValue("user")).getValue("login"));
            Assert.assertEquals("Commercial", comUser1.getValue("role"));
            ModelAwareRepeaterEntry comUser2 = comUsers.getEntry(2);
            Assert.assertEquals("gubersfeld", ((ContentValue) comUser2.getValue("user")).getValue("login"));
            Assert.assertEquals("Commercial", comUser2.getValue("role"));

            // Projects
            ModelAwareRepeater projectsUsers = projects.getRepeater("users");
            Assert.assertEquals(2, projectsUsers.getSize());
            ModelAwareRepeaterEntry projectsUser1 = projectsUsers.getEntry(1);
            Assert.assertEquals("lperier", ((ContentValue) projectsUser1.getValue("user")).getValue("login"));
            Assert.assertFalse(projectsUser1.hasValue("role"));
            ModelAwareRepeaterEntry projectsUser2 = projectsUsers.getEntry(2);
            Assert.assertEquals("fravetier", ((ContentValue) projectsUser2.getValue("user")).getValue("login"));
            Assert.assertEquals("Directeur des projets", projectsUser2.getValue("role"));

            // Games
            ModelAwareRepeater gamesUsers = games.getRepeater("users");
            Assert.assertEquals(1, gamesUsers.getSize());
            ModelAwareRepeaterEntry gamesUser1 = gamesUsers.getEntry(1);
            Assert.assertEquals("nportman", ((ContentValue) gamesUser1.getValue("user")).getValue("login"));
            Assert.assertEquals("Actrice", gamesUser1.getValue("role"));
        }
    }
    
    /**
     * Test the users with no role on org units
     * @param orgUnits the synchronized org units
     * @param nbLangs the number of languages
     */
    protected void _testUsersWithNoRoleOnOrgUnits(List<? extends Content> orgUnits, int nbLangs)
    {
        for (int i = 0; i < nbLangs; i++)
        {
            int offset = 0;
            Content ametys = orgUnits.get(offset++ * nbLangs + i);
            Content dev = orgUnits.get(offset++ * nbLangs + i);
            Content com = orgUnits.get(offset++ * nbLangs + i);
            Content projects = orgUnits.get(offset++ * nbLangs + i);
            Content games = orgUnits.get(offset++ * nbLangs + i);
            
            // Ametys
            ModelAwareRepeater ametysUsers = ametys.getRepeater("users");
            Assert.assertEquals(2, ametysUsers.getSize());
            ModelAwareRepeaterEntry ametysUser1 = ametysUsers.getEntry(1);
            Assert.assertEquals("cdamioli", ((ContentValue) ametysUser1.getValue("user")).getValue("login"));
            ModelAwareRepeaterEntry ametysUser2 = ametysUsers.getEntry(2);
            Assert.assertEquals("gubersfeld", ((ContentValue) ametysUser2.getValue("user")).getValue("login"));
            
            // Dev
            ModelAwareRepeater devUsers = dev.getRepeater("users");
            Assert.assertEquals(4, devUsers.getSize());
            ModelAwareRepeaterEntry devUser1 = devUsers.getEntry(1);
            Assert.assertEquals("lbouchet", ((ContentValue) devUser1.getValue("user")).getValue("login"));
            ModelAwareRepeaterEntry devUser2 = devUsers.getEntry(2);
            Assert.assertEquals("sprieul", ((ContentValue) devUser2.getValue("user")).getValue("login"));
            ModelAwareRepeaterEntry devUser3 = devUsers.getEntry(3);
            Assert.assertEquals("laumeunier", ((ContentValue) devUser3.getValue("user")).getValue("login"));
            ModelAwareRepeaterEntry devUser4 = devUsers.getEntry(4);
            Assert.assertEquals("glefebvre", ((ContentValue) devUser4.getValue("user")).getValue("login"));
            
            // Com
            ModelAwareRepeater comUsers = com.getRepeater("users");
            Assert.assertEquals(2, comUsers.getSize());
            ModelAwareRepeaterEntry comUser1 = comUsers.getEntry(1);
            Assert.assertEquals("jbourak", ((ContentValue) comUser1.getValue("user")).getValue("login"));
            ModelAwareRepeaterEntry comUser2 = comUsers.getEntry(2);
            Assert.assertEquals("gubersfeld", ((ContentValue) comUser2.getValue("user")).getValue("login"));
            
            // Projects
            ModelAwareRepeater projectsUsers = projects.getRepeater("users");
            Assert.assertEquals(2, projectsUsers.getSize());
            ModelAwareRepeaterEntry projectsUser1 = projectsUsers.getEntry(1);
            Assert.assertEquals("lperier", ((ContentValue) projectsUser1.getValue("user")).getValue("login"));
            ModelAwareRepeaterEntry projectsUser2 = projectsUsers.getEntry(2);
            Assert.assertEquals("fravetier", ((ContentValue) projectsUser2.getValue("user")).getValue("login"));
    
            // Games
            ModelAwareRepeater gamesUsers = games.getRepeater("users");
            Assert.assertEquals(1, gamesUsers.getSize());
            ModelAwareRepeaterEntry gamesUser1 = gamesUsers.getEntry(1);
            Assert.assertEquals("nportman", ((ContentValue) gamesUser1.getValue("user")).getValue("login"));
        }
        
        // Check that there is no role on each org unit's users
        for (Content orgUnit : orgUnits)
        {
            Repeater users = orgUnit.getRepeater("users");
            for (RepeaterEntry user : users.getEntries())
            {
                Assert.assertFalse(user.hasValue("role"));
            }
        }
    }
    
    /**
     * Test the new org units
     * @param orgUnits the synchronized org units
     * @param nbLangs the number of languages
     */
    protected void _testNewOrgUnits(List<? extends Content> orgUnits, int nbLangs)
    {
        Assert.assertEquals(4 * nbLangs, orgUnits.size());
    
        for (int i = 0; i < nbLangs; i++)
        {
            int offset = 0;
            Content ametys = orgUnits.get(offset++ * nbLangs + i);
            Content dev = orgUnits.get(offset++ * nbLangs + i);
            Content com = orgUnits.get(offset++ * nbLangs + i);
            Content projects = orgUnits.get(offset++ * nbLangs + i);
    
            // Ametys
            Assert.assertEquals("1", ametys.getInternalDataHolder().getValue("orgunit-remote-id"));
            Assert.assertEquals("1", ametys.getValue("orgUnitId"));
            Assert.assertEquals("Pôle Ametys", ametys.getValue("title"));
            Assert.assertEquals("Pôle Ametys", ametys.getValue("abstract"));
            Content parent = _orgChartPageHandler.getParentContent(ametys);
            Assert.assertNull(parent);
            List<Content> childContents = _orgChartPageHandler.getChildContents(ametys);
            Assert.assertEquals(1, childContents.size());
            Assert.assertTrue(childContents.contains(dev));
    
            // Dev
            Assert.assertEquals("2", dev.getInternalDataHolder().getValue("orgunit-remote-id"));
            Assert.assertEquals("2", dev.getValue("orgUnitId"));
            Assert.assertEquals("Service développement", dev.getValue("title"));
            Assert.assertEquals("Service développement", dev.getValue("abstract"));
            parent = _orgChartPageHandler.getParentContent(dev);
            Assert.assertEquals(ametys, parent);
            childContents = _orgChartPageHandler.getChildContents(dev);
            Assert.assertEquals(1, childContents.size());
            Assert.assertTrue(childContents.contains(com));
    
            // Com
            Assert.assertEquals("3", com.getInternalDataHolder().getValue("orgunit-remote-id"));
            Assert.assertEquals("3", com.getValue("orgUnitId"));
            Assert.assertEquals("Service commercial", com.getValue("title"));
            Assert.assertEquals("Service commercial", com.getValue("abstract"));
            parent = _orgChartPageHandler.getParentContent(com);
            Assert.assertEquals(dev, parent);
            childContents = _orgChartPageHandler.getChildContents(com);
            Assert.assertTrue(childContents.isEmpty());
    
            // Projects
            Assert.assertEquals("4", projects.getInternalDataHolder().getValue("orgunit-remote-id"));
            Assert.assertEquals("4", projects.getValue("orgUnitId"));
            Assert.assertEquals("Service des projets", projects.getValue("title"));
            Assert.assertEquals("Service des projets", projects.getValue("abstract"));
            parent = _orgChartPageHandler.getParentContent(projects);
            Assert.assertNull(parent);
            childContents = _orgChartPageHandler.getChildContents(projects);
            Assert.assertTrue(childContents.isEmpty());
        }
    }
    
    /**
     * Test the new org units
     * @param orgUnits the synchronized org units
     * @param nbLangs the number of languages
     */
    protected void _testNewUsersOnOrgUnits(List<? extends Content> orgUnits, int nbLangs)
    {
        for (int i = 0; i < nbLangs; i++)
        {
            int offset = 0;
            Content ametys = orgUnits.get(offset++ * nbLangs + i);
            Content dev = orgUnits.get(offset++ * nbLangs + i);
            Content com = orgUnits.get(offset++ * nbLangs + i);
            Content projects = orgUnits.get(offset++ * nbLangs + i);
            
            // Ametys
            ModelAwareRepeater ametysUsers = ametys.getRepeater("users");
            Assert.assertEquals(2, ametysUsers.getSize());
            ModelAwareRepeaterEntry ametysUser1 = ametysUsers.getEntry(1);
            Assert.assertEquals("cdamioli", ((ContentValue) ametysUser1.getValue("user")).getValue("login"));
            Assert.assertEquals("President", ametysUser1.getValue("role"));
            ModelAwareRepeaterEntry ametysUser2 = ametysUsers.getEntry(2);
            Assert.assertEquals("gubersfeld", ((ContentValue) ametysUser2.getValue("user")).getValue("login"));
            Assert.assertEquals("Directeur commercial 2", ametysUser2.getValue("role"));
            
            // Dev
            ModelAwareRepeater devUsers = dev.getRepeater("users");
            Assert.assertEquals(3, devUsers.getSize());
            ModelAwareRepeaterEntry devUser1 = devUsers.getEntry(1);
            Assert.assertEquals("sprieul", ((ContentValue) devUser1.getValue("user")).getValue("login"));
            Assert.assertFalse(devUser1.hasValue("role"));
            ModelAwareRepeaterEntry devUser2 = devUsers.getEntry(2);
            Assert.assertEquals("laumeunier", ((ContentValue) devUser2.getValue("user")).getValue("login"));
            Assert.assertEquals("Chef du developpement", devUser2.getValue("role"));
            ModelAwareRepeaterEntry devUser3 = devUsers.getEntry(3);
            Assert.assertEquals("glefebvre", ((ContentValue) devUser3.getValue("user")).getValue("login"));
            Assert.assertFalse(devUser3.hasValue("role"));
            
            // Com
            ModelAwareRepeater comUsers = com.getRepeater("users");
            Assert.assertEquals(2, comUsers.getSize());
            ModelAwareRepeaterEntry comUser1 = comUsers.getEntry(1);
            Assert.assertEquals("jbourak", ((ContentValue) comUser1.getValue("user")).getValue("login"));
            Assert.assertEquals("Commercial 2", comUser1.getValue("role"));
            ModelAwareRepeaterEntry comUser2 = comUsers.getEntry(2);
            Assert.assertEquals("sprieul", ((ContentValue) comUser2.getValue("user")).getValue("login"));
            Assert.assertFalse(comUser2.hasValue("role"));
            
            // Projects
            ModelAwareRepeater projectsUsers = projects.getRepeater("users");
            Assert.assertEquals(2, projectsUsers.getSize());
            ModelAwareRepeaterEntry projectsUser1 = projectsUsers.getEntry(1);
            Assert.assertEquals("lperier", ((ContentValue) projectsUser1.getValue("user")).getValue("login"));
            Assert.assertFalse(projectsUser1.hasValue("role"));
            ModelAwareRepeaterEntry projectsUser2 = projectsUsers.getEntry(2);
            Assert.assertEquals("fravetier", ((ContentValue) projectsUser2.getValue("user")).getValue("login"));
            Assert.assertEquals("Directeur des projets", projectsUser2.getValue("role"));
        }
    }
    
    /**
     * Runs given script files
     * @param scripts THe list of script files
     * @throws Exception if an error occurs
     */
    protected void _updateScript(List<File> scripts) throws Exception
    {
        Connection connection = null;
        
        try
        {
            connection = ConnectionHelper.getConnection("SQL-test");
            
            for (File script : scripts)
            {
                SQLScriptHelper.runScript(connection, new FileInputStream(script));
            }
        }
        finally 
        {
            ConnectionHelper.cleanup(connection);
        }
    }
    
    private List<String> _getLanguages()
    {
        List<String> languages = new ArrayList<>();
        languages.add("fr");
        languages.add("it");
        return languages;
    }
}
