/*
 *  Copyright 2017 Anyware Services
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.ametys.plugins.userdirectory;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.junit.After;
import org.junit.Assert;
import org.junit.Test;

import org.ametys.cms.content.indexing.solr.observation.ObserverHelper;
import org.ametys.cms.repository.ModifiableContent;
import org.ametys.core.datasource.ConnectionHelper;
import org.ametys.core.script.SQLScriptHelper;
import org.ametys.core.user.UserIdentity;
import org.ametys.core.user.population.UserPopulationDAO;
import org.ametys.plugins.contentio.synchronize.SynchronizableContentsCollection;
import org.ametys.plugins.contentio.synchronize.SynchronizableContentsCollectionDAO;
import org.ametys.plugins.contentio.synchronize.impl.DefaultSynchronizingContentOperator;
import org.ametys.runtime.test.framework.AbstractRuntimeTestCase;
import org.ametys.runtime.test.framework.CocoonTestCase;
import org.ametys.runtime.test.framework.CocoonTestCase.AllFeatures;
import org.ametys.runtime.test.framework.TestConfigSet;

import com.google.common.collect.ImmutableList;

/**
 * Test cases for SCC from CONTENT-IO.
 */
@AllFeatures
public class SCCUsersTestCase extends AbstractRuntimeTestCase implements CocoonTestCase
{
    /** The SCC DAO */
    protected static SynchronizableContentsCollectionDAO _sccDAO;
    
    /** The User Population DAO */
    protected static UserPopulationDAO _userPopulationDAO;

    private static String _userPopulationId;
    
    @Override
    public String getDBType()
    {
        return ConnectionHelper.DATABASE_MYSQL;
    }
    
    @Override
    public File[] getStartScriptFiles()
    {
        return new File[] {
            new File("test/environments/scripts/jdbc/" + getDBType() + "/user/deleteTables.sql"),
            new File("test/environments/scripts/jdbc/" + getDBType() + "/user/initTables.sql")
        };
    }
    
    @Override
    public TestConfigSet getConfigSet()
    {
        return CocoonTestCase.super.getConfigSet()
                .withRuntimeFile("test/environments/runtime.xml")
                .withConfigFile("test/environments/config.xml")
                .withLdapFile("test/environments/datasources/datasource-ldap.xml")
                .withWebappPath("test/environments/webapp");
    }
    
    @Override
    protected void essentialsBefore() throws Exception
    {
        ObserverHelper.suspendObservationForIndexation();
        super.essentialsBefore();
    }
    
    @Override
    public void lookupComponents(ServiceManager manager) throws ServiceException
    {
        _sccDAO = (SynchronizableContentsCollectionDAO) manager.lookup(SynchronizableContentsCollectionDAO.ROLE);
        _userPopulationDAO = (UserPopulationDAO) manager.lookup(UserPopulationDAO.ROLE);
    }
    
    @Override
    public void prepareComponents() throws Exception
    {
        _createPopulation();
    }
    
    /**
     * Create population for tests
     */
    protected void _createPopulation()
    {
        /* User directory LDAP */
        String modelId = "org.ametys.plugins.core.user.directory.Ldap";
        Map<String, Object> userDirectoryLdapParameters = new HashMap<>();
        userDirectoryLdapParameters.put("id", "contentio-test-ud-ldap");
        userDirectoryLdapParameters.put("udModelId", modelId);
        userDirectoryLdapParameters.put(modelId + "$" + "runtime.users.ldap.datasource", "LDAP-userdirectory");
        userDirectoryLdapParameters.put(modelId + "$" + "runtime.users.ldap.peopleDN", "ou=people");
        userDirectoryLdapParameters.put(modelId + "$" + "runtime.users.ldap.baseFilter", "(objectclass=*)");
        userDirectoryLdapParameters.put(modelId + "$" + "runtime.users.ldap.scope", "one");
        userDirectoryLdapParameters.put(modelId + "$" + "runtime.users.ldap.loginAttr", "uid");
        userDirectoryLdapParameters.put(modelId + "$" + "runtime.users.ldap.firstnameAttr", "givenName");
        userDirectoryLdapParameters.put(modelId + "$" + "runtime.users.ldap.lastnameAttr", "sn");
        userDirectoryLdapParameters.put(modelId + "$" + "runtime.users.ldap.emailAttr", "mail");
        userDirectoryLdapParameters.put(modelId + "$" + "runtime.users.ldap.emailMandatory", "false");
        userDirectoryLdapParameters.put(modelId + "$" + "runtime.users.ldap.serverSideSorting", "true");

        /* User directory JDBC */
        modelId = "org.ametys.plugins.core.user.directory.Jdbc";
        Map<String, Object> userDirectorySqlParameters = new HashMap<>();
        userDirectorySqlParameters.put("id", "contentio-test-ud-sql");
        userDirectorySqlParameters.put("udModelId", modelId);
        userDirectorySqlParameters.put(modelId + "$" + "runtime.users.jdbc.datasource", "SQL-test");
        userDirectorySqlParameters.put(modelId + "$" + "runtime.users.jdbc.table", "Users");
        userDirectorySqlParameters.put(modelId + "$" + "runtime.users.jdbc.strong.password", false);
        userDirectorySqlParameters.put(modelId + "$" + "runtime.users.jdbc.strong.password.min-length", 8);

        /* Credential provider */
        modelId = "org.ametys.core.authentication.Defined";
        Map<String, Object> credentialProviderParameters = new HashMap<>();
        credentialProviderParameters.put("id", "contentio-test-credential");
        credentialProviderParameters.put("cpModelId", modelId);
        credentialProviderParameters.put(modelId + "$" + "runtime.authentication.form.security.level", "low");
        credentialProviderParameters.put(modelId + "$" + "runtime.authentication.form.security.storage", "SQL-test");

        /* Create user population */
        _userPopulationId = "contentio-test-population";
        Map<String, Object> resultMap = _userPopulationDAO.add(_userPopulationId, "Mixed Population", ImmutableList.of(userDirectoryLdapParameters, userDirectorySqlParameters), Collections.singletonList(credentialProviderParameters));
        Assert.assertEquals(_userPopulationId, resultMap.get("id"));
    }
    
    /**
     * After each test, clean
     */
    @After
    public void sccAfterTest()
    {
        if (isPluginsManagerOk())
        {
            _removeScc();
        }
    }
    
    private void _removeScc()
    {
        List<SynchronizableContentsCollection> collections = _sccDAO.getSynchronizableContentsCollections();
        for (SynchronizableContentsCollection collection : collections)
        {
            String id = collection.getId();
            try
            {
                collection.empty(_logger);
                _sccDAO.removeCollection(id);
                _logger.info("Collection '{}' was successfully removed.", id);
            }
            catch (Exception e)
            {
                _logger.error("An error occured when trying to remove collection '{}'", id, e);
            }
        }
    }
    
    /**
     * Test a User Population SCC
     * @throws Exception if an error occurs
     */
    @Test
    public void testUserPopulation() throws Exception
    {
        /* Count available languages */
        List<String> languages = _getLanguages();
        int nbLangs = languages.size();

        /* Create the SCC */
        Map<String, Object> sccParameters = new HashMap<>();
        sccParameters.put("label", "Collection TEST");
        sccParameters.put("contentType", "agent");
        sccParameters.put("contentPrefix", "test-");
        sccParameters.put("workflowName", "user");
        sccParameters.put("initialActionId", 11);
        sccParameters.put("synchronizeActionId", 800);
        sccParameters.put("validateActionId", 41);
        sccParameters.put("contentOperator", DefaultSynchronizingContentOperator.class.getName());
        sccParameters.put("languages", languages);
        sccParameters.put("modelId", "org.ametys.plugins.userdirectory.synchronize.collection.userPopulation");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.collection.userPopulation" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "populationId", _userPopulationId);
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.collection.userPopulation" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "firstname", "firstname");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.collection.userPopulation" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "lastname", "lastname");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.collection.userPopulation" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "email", "mail");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.collection.userPopulation" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "additionalSearchFilter", "(givenName=Laurence)"); // FIXME CONTENTIO-102
    
        String collectionId = _sccDAO.addCollection(sccParameters);
        Assert.assertNotNull(collectionId);
        
        SynchronizableContentsCollection scc = _sccDAO.getSynchronizableContentsCollection(collectionId);
        Assert.assertNotNull(scc);
        
        /* Populate with the SCC */
        List<ModifiableContent> contents = scc.populate(_logger);
        Assert.assertNotNull(contents);
        Assert.assertEquals(nbLangs * 4, contents.size()); // FIXME CONTENTIO-102
        
        Map<String, Integer> result = scc.getSynchronizationResult();
        Assert.assertEquals(Integer.valueOf(nbLangs * 4), result.get(SynchronizableContentsCollection.RESULT_NB_CREATED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_SYNCHRONIZED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_NOT_CHANGED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_DELETED_CONTENTS));
    }
    
    /**
     * Test an user LDAP SCC
     * @throws Exception if an error occurs
     */
    @Test
    public void testUserLDAP() throws Exception
    {
        /* Count available languages */
        List<String> languages = _getLanguages();
        int nbLangs = languages.size();

        /* Create the SCC */
        Map<String, Object> sccParameters = new HashMap<>();
        sccParameters.put("label", "Collection TEST");
        sccParameters.put("contentType", "agent");
        sccParameters.put("contentPrefix", "test-");
        sccParameters.put("workflowName", "user");
        sccParameters.put("initialActionId", 11);
        sccParameters.put("synchronizeActionId", 800);
        sccParameters.put("validateActionId", 41);
        sccParameters.put("contentOperator", DefaultSynchronizingContentOperator.class.getName());
        sccParameters.put("languages", languages);
        sccParameters.put("ignoreRestrictions", false);
        sccParameters.put("modelId", "org.ametys.plugins.userdirectory.synchronize.users.ldap");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.users.ldap" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "datasourceId", "LDAP-userdirectory");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.users.ldap" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "peopleDN", "ou=people");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.users.ldap" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "baseFilter", "(objectclass=*)");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.users.ldap" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "scope", "one");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.users.ldap" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "login", "uid");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.users.ldap" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "populationId", _userPopulationId);
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.users.ldap" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "mapping", "["
            + "{\"metadata-ref\":\"firstname\",\"attribute\":\"givenName\",\"synchro\":true},"
            + "{\"metadata-ref\":\"lastname\",\"attribute\":\"sn\",\"synchro\":true},"
            + "{\"metadata-ref\":\"title\",\"attribute\":\"title\",\"synchro\":true}]");
        
        String collectionId = _sccDAO.addCollection(sccParameters);
        Assert.assertNotNull(collectionId);
        
        SynchronizableContentsCollection scc = _sccDAO.getSynchronizableContentsCollection(collectionId);
        Assert.assertNotNull(scc);
        
        /* Import a content */
        List<ModifiableContent> contents = scc.importContent("bmaurel", null, _logger); // FIXME CONTENTIO-102
        Assert.assertNotNull(contents);
        Assert.assertEquals(nbLangs, contents.size());
        
        Map<String, Integer> result = scc.getSynchronizationResult();
        Assert.assertEquals(Integer.valueOf(nbLangs), result.get(SynchronizableContentsCollection.RESULT_NB_CREATED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_SYNCHRONIZED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_NOT_CHANGED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_DELETED_CONTENTS));
        
        ModifiableContent firstContent = contents.get(0);
        _testSyncUser(firstContent, "bmaurel", "bmaurel", "Bérénice");

        /* Update the sync code of a content */
        scc.updateSyncInformations(firstContent, "ced", _logger); // FIXME CONTENTIO-102
        Assert.assertEquals("ced", firstContent.getValue(scc.getIdField())); // FIXME CONTENTIO-102

        /* Update the modified content */
        scc.synchronizeContent(firstContent, _logger);
        result = scc.getSynchronizationResult();
        Assert.assertEquals(Integer.valueOf(nbLangs), result.get(SynchronizableContentsCollection.RESULT_NB_CREATED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(1), result.get(SynchronizableContentsCollection.RESULT_NB_SYNCHRONIZED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_NOT_CHANGED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_DELETED_CONTENTS));
        
        _testSyncUser(firstContent, "bmaurel", "ced", "Cedric");
    }
    
    /**
     * Test an user SQL SCC
     * @throws Exception if an error occurs
     */
    @Test
    public void testUserSQL() throws Exception
    {
        /* Count available languages */
        List<String> languages = _getLanguages();
        int nbLangs = languages.size();

        /* Create the SCC */
        Map<String, Object> sccParameters = new HashMap<>();
        sccParameters.put("label", "Collection TEST");
        sccParameters.put("contentType", "agent");
        sccParameters.put("contentPrefix", "test-");
        sccParameters.put("workflowName", "user");
        sccParameters.put("initialActionId", 11);
        sccParameters.put("synchronizeActionId", 800);
        sccParameters.put("validateActionId", 41);
        sccParameters.put("contentOperator", DefaultSynchronizingContentOperator.class.getName());
        sccParameters.put("languages", languages);
//        sccParameters.put("removalSync", true);
        sccParameters.put("ignoreRestrictions", false);
        sccParameters.put("modelId", "org.ametys.plugins.userdirectory.synchronize.users.sql");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.users.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "datasourceId", "SQL-test");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.users.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "tableName", "synchro_sql_users");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.users.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "login", "login");
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.users.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "populationId", _userPopulationId);
        sccParameters.put("org.ametys.plugins.userdirectory.synchronize.users.sql" + SynchronizableContentsCollectionDAO.SCC_PARAMETERS_SEPARATOR + "mapping", "["
            + "{\"metadata-ref\":\"firstname\",\"attribute\":\"prenom\",\"synchro\":false},"
            + "{\"metadata-ref\":\"lastname\",\"attribute\":\"nom\",\"synchro\":false},"
            + "{\"metadata-ref\":\"title\",\"attribute\":\"title\",\"synchro\":false}]");
        
        String collectionId = _sccDAO.addCollection(sccParameters);
        Assert.assertNotNull(collectionId);
        
        SynchronizableContentsCollection scc = _sccDAO.getSynchronizableContentsCollection(collectionId);
        Assert.assertNotNull(scc);

        /* Populate with the SCC */
        List<ModifiableContent> contents = scc.populate(_logger);
        Assert.assertNotNull(contents);
        Assert.assertEquals(nbLangs * 4, contents.size());
        Map<String, Integer> result = scc.getSynchronizationResult();
        Assert.assertEquals(Integer.valueOf(nbLangs * 4), result.get(SynchronizableContentsCollection.RESULT_NB_CREATED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_SYNCHRONIZED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_NOT_CHANGED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_DELETED_CONTENTS));
        
        /* Test synchronized users data */
        _testUsersData(contents, nbLangs);
        
        List<File> files = new ArrayList<>();
        files.add(new File("test/environments/scripts/jdbc/" + getDBType() + "/user/deleteUserTables.sql"));
        files.add(new File("test/environments/scripts/jdbc/" + getDBType() + "/user/initNewUserTables.sql"));
        _updateScript(files);
        
        /* Populate with the SCC */
        contents = scc.populate(_logger);
        Assert.assertNotNull(contents);
        Assert.assertEquals(nbLangs * 2, contents.size());
        result = scc.getSynchronizationResult();
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_CREATED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(nbLangs), result.get(SynchronizableContentsCollection.RESULT_NB_SYNCHRONIZED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(nbLangs), result.get(SynchronizableContentsCollection.RESULT_NB_NOT_CHANGED_CONTENTS));
        Assert.assertEquals(Integer.valueOf(0), result.get(SynchronizableContentsCollection.RESULT_NB_DELETED_CONTENTS));
        
        _testNewUsersData(contents, nbLangs);
    }
    
    /**
     * Test synchronized contents data
     * @param contents the synchronized contents
     * @param nbLangs number of languages
     * @throws IOException if an error occurred
     */
    protected void _testUsersData(List<ModifiableContent> contents, int nbLangs) throws IOException
    {
        for (int i = 0; i < nbLangs; i++)
        {
            int offset = 0;
            
            ModifiableContent loic = contents.get(offset++ * nbLangs + i);
            _testSyncUser(loic, "Loic Bouchet", "loic", "Loic");

            ModifiableContent simon = contents.get(offset++ * nbLangs + i);
            _testSyncUser(simon, "Simon Prieul", "bmaurel", "Simon");
            
            ModifiableContent natalie = contents.get(offset++ * nbLangs + i);
            _testSyncUser(natalie, "Natalie Portman", "bob", "Natalie");
            
            ModifiableContent barack = contents.get(offset++ * nbLangs + i);
            _testSyncUser(barack, "Barack Obama", "ced", "Barack");
        }
    }
    
    /**
     * Test synchronized contents data
     * @param contents the synchronized contents
     * @param nbLangs number of languages
     * @throws IOException if an error occurred
     */
    protected void _testNewUsersData(List<ModifiableContent> contents, int nbLangs) throws IOException
    {
        for (int i = 0; i < nbLangs; i++)
        {
            int offset = 0;
            
            ModifiableContent loic = contents.get(offset++ * nbLangs + i);
            _testSyncUser(loic, "Gerard Bouchard", "loic", "Loic");
            
            ModifiableContent natalie = contents.get(offset++ * nbLangs + i);
            _testSyncUser(natalie, "Natalie Portman", "bob", "Natalie");
        }
    }

    /**
     * Test data from user
     * @param userContent the user content
     * @param title the user title
     * @param login the login
     * @param firstname the firstname
     * @throws IOException if an error occurred
     */
    protected void _testSyncUser(ModifiableContent userContent, String title, String login, String firstname) throws IOException
    {
        Assert.assertEquals(title, userContent.getValue("title"));
        Assert.assertEquals(firstname, userContent.getValue("firstname"));
        
        // lastname has write restrictions so it is not taken into account and "ignoreRestrictions" is set to false
        Assert.assertFalse(userContent.hasValue("lastname"));

        UserIdentity user = userContent.getValue("user");
        Assert.assertNotNull(user);
        Assert.assertEquals(login, user.getLogin());
        Assert.assertEquals(_userPopulationId, user.getPopulationId());
    }
    
    private List<String> _getLanguages()
    {
        List<String> languages = new ArrayList<>();
        languages.add("fr");
        languages.add("it");
        return languages;
    }
    
    /**
     * Runs given script files
     * @param scripts THe list of script files
     * @throws Exception if an error occurs
     */
    protected void _updateScript(List<File> scripts) throws Exception
    {
        Connection connection = null;
        
        try
        {
            connection = ConnectionHelper.getConnection("SQL-test");
            
            for (File script : scripts)
            {
                SQLScriptHelper.runScript(connection, new FileInputStream(script));
            }
        }
        finally 
        {
            ConnectionHelper.cleanup(connection);
        }
    }
}
